/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the GNOME helper app implementation.
 *
 * The Initial Developer of the Original Code is
 * IBM Corporation.
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *  Brian Ryner <bryner@brianryner.com>  (Original Author)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#include "nsKDERegistry.h"
#include "prlink.h"
#include "prmem.h"
#include "nsString.h"
#include "nsILocalFile.h"
#include "nsMIMEInfoUnix.h"
#include "nsAutoPtr.h"
#include "nsKDEUtils.h"

/* static */ PRBool
nsKDERegistry::HandlerExists(const char *aProtocolScheme)
{
    nsCStringArray command;
    command.AppendCString( NS_LITERAL_CSTRING( "HANDLEREXISTS" ));
    command.AppendCString( nsCAutoString( aProtocolScheme ));
    return nsKDEUtils::command( command );
}

/* static */ nsresult
nsKDERegistry::LoadURL(nsIURI *aURL)
{
    nsCStringArray command;
    command.AppendCString( NS_LITERAL_CSTRING( "OPEN" ));
    nsCString url;
    aURL->GetSpec( url );
    command.AppendCString( url );
    return nsKDEUtils::command( command );
}

/* static */ void
nsKDERegistry::GetAppDescForScheme(const nsACString& aScheme,
                                     nsAString& aDesc)
{
    nsCStringArray command;
    command.AppendCString( NS_LITERAL_CSTRING( "GETAPPDESCFORSCHEME" ));
    command.AppendCString( aScheme );
    nsCStringArray output;
    if( nsKDEUtils::command( command, &output ) && output.Count() == 1 )
        CopyUTF8toUTF16( *output[ 0 ], aDesc );
}


/* static */ already_AddRefed<nsMIMEInfoBase>
nsKDERegistry::GetFromExtension(const nsACString& aFileExt)
{
    NS_ASSERTION(aFileExt[0] != '.', "aFileExt shouldn't start with a dot");
    nsCStringArray command;
    command.AppendCString( NS_LITERAL_CSTRING( "GETFROMEXTENSION" ));
    command.AppendCString( aFileExt );
    return GetFromHelper( command );
}

/* static */ already_AddRefed<nsMIMEInfoBase>
nsKDERegistry::GetFromType(const nsACString& aMIMEType)
{
    nsCStringArray command;
    command.AppendCString( NS_LITERAL_CSTRING( "GETFROMTYPE" ));
    command.AppendCString( aMIMEType );
    return GetFromHelper( command );
}

/* static */ already_AddRefed<nsMIMEInfoBase>
nsKDERegistry::GetFromHelper(const nsCStringArray& command)
{
    nsCStringArray output;
    if( nsKDEUtils::command( command, &output ) && output.Count() == 3 )
        {
        nsCString mimetype = *output[ 0 ];
        nsRefPtr<nsMIMEInfoUnix> mimeInfo = new nsMIMEInfoUnix( mimetype );
        NS_ENSURE_TRUE(mimeInfo, nsnull);
        nsCString description = *output[ 1 ];
        mimeInfo->SetDescription(NS_ConvertUTF8toUTF16(description));
        nsCString handlerAppName = *output[ 2 ];
        mimeInfo->SetDefaultDescription(NS_ConvertUTF8toUTF16(handlerAppName));
        mimeInfo->SetPreferredAction(nsIMIMEInfo::useSystemDefault);
        nsMIMEInfoBase* retval;
        NS_ADDREF((retval = mimeInfo));
        return retval;
        }
    return nsnull;
}
